"""
CtrlCraftProject module
"""
from __future__ import absolute_import

from builtins import object
import os
import re
from shutil import copyfile

from qgis.core import (QgsProject)


class CtrlCraftProject(object):

    def __init__(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        qgis_dirname = os.path.join(dirname, 'qgis')
        viewer_3d_dirname = os.path.join(dirname, 'widgets', 'viewer_3d')

        self.__template = os.path.join(qgis_dirname, 'template.qgs')
        self.__color_3d_template = os.path.join(viewer_3d_dirname, 'settings_data.py.ref')
        qgs_project = os.path.join(qgis_dirname, 'ctrlcraft.qgs')
        settings_data = os.path.join(viewer_3d_dirname, 'settings_data.py')
        self.__project = os.path.abspath(qgs_project)  # for '\' on Windows
        self.__color_3d = os.path.abspath(settings_data)

    def equal(self, filename):
        return self.__project == filename

    def exists(self):
        return os.path.isfile(self.__project)

    def create(self):
        if not self.exists():
            copyfile(self.__template, self.__project)
            copyfile(self.__color_3d_template, self.__color_3d)

    def reset(self):
        self.remove()
        self.create()

    def close(self):
        QgsProject.instance().removeAllMapLayers()
        QgsProject.instance().clear()

    def load(self):
        QgsProject.instance().setFileName(self.__project)
        QgsProject.instance().read()

    def remove(self):
        QgsProject.instance().removeAllMapLayers()
        QgsProject.instance().clear()
        os.remove(self.__project)

    def is_loaded(self):
        qgs_project = os.path.abspath(QgsProject.instance().fileName())
        return qgs_project == self.__project

    @staticmethod
    def layer(name):
        """
        Returns the corresponding vector layer.
        """

        if not CtrlCraftProject().is_loaded():
            return None

        for lyr in list(QgsProject.instance().mapLayers().values()):
            source = lyr.source()
            if source[:6] == 'dbname':
                source = source.replace('\'', '"')
                vals = dict(re.findall('(\S+)="?(.*?)"? ', source))
                table = vals['table'].split('.')
                schemaName = table[0].strip('"')
                tableName = table[1].strip('"')
                table = "{}.{}".format(schemaName, tableName)

                if table == name:
                    return lyr

        return None

    @staticmethod
    def slabsLayer():
        """
        Returns
        """

        return CtrlCraftProject.layer('ctrlcraft.slabs_available')

    @staticmethod
    def probeLayer():
        """
        Returns
        """

        return CtrlCraftProject.layer('ctrlcraft.probe_last_positions')

    @staticmethod
    def topoLayer():
        """
        Returns
        """

        return CtrlCraftProject.layer('ctrlcraft.topo_control')

    @staticmethod
    def estimatedBlocsLayer():
        """
        Returns
        """

        return CtrlCraftProject.layer('ctrlcraft.estimated_blocs')

    @staticmethod
    def co3Layer():
        """
        Returns
        """

        return CtrlCraftProject.layer('ctrlcraft.co3')

    @staticmethod
    def levelsLayer():
        """
        Returns
        """

        return CtrlCraftProject.layer('ctrlcraft.levels')

    @staticmethod
    def setDepositGroupName(name):
        """
        """

        children = QgsProject.instance().layerTreeRoot().children()

        if children:
            children[0].setName(name)
