# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartEstimatedBlocs
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os
import csv
import configparser

from PyQt5 import uic
from PyQt5.QtWidgets import (QFileDialog,
                         qApp,
                         QLabel,
                         QDialog)

from qgis.gui import QgsMessageBar

from CtrlCraft import CtrlCraftLogger

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'export_db.ui'))


class CtrlCraftDatabaseExport(QDialog, FORM_CLASS):
    """
    Dialog to import Slabs in database.
    """

    def __init__(self, data, projIni, deposit, parent=None):
        super(CtrlCraftDatabaseExport, self).__init__(parent)
        self.setupUi(self)
        self.projIni = projIni
        self.deposit = deposit
        self.data = data
        self.config = configparser.ConfigParser()
        self.config.read(self.projIni)

        #self.__populateProjComboBox()
        self.mCancelButton.clicked.connect(self.__close)
        self.mImportButton.setText( self.tr('Export') )
        self.mImportButton.clicked.connect(self.__export)
        self.mSelectButton.clicked.connect(self.__select)

        self.dep_not_in_conf = deposit not in self.config
        self.mProjInfo.setVisible(self.dep_not_in_conf)
        self.mNSRadioButton.setHidden(self.dep_not_in_conf)
        self.mWGSRadioButton.setHidden(self.dep_not_in_conf)

        self.mProgressBar.setValue(0)
        self.mLabel.setText('')

    def __close(self):
        self.close()

    def __select(self):
        filename, _ = QFileDialog.getSaveFileName(self, self.tr("Dump file"), filter="*.dump")

        if filename:
            self.mLineEdit.setText(filename)

    def __export(self):
        filename = self.mLineEdit.text()
        if not filename:
            return

        deposit = self.deposit if self.deposit != None else 'DEFAULT'
        self.mSelectButton.setEnabled(False)
        self.mCancelButton.setEnabled(False)
        self.mImportButton.setEnabled(False)

        self.mLabel.setText(self.tr('Export database...'))
        self.mProgressBar.setMinimum(0)
        self.mProgressBar.setMaximum(0)
        qApp.processEvents()
        if not self.dep_not_in_conf and self.mWGSRadioButton.isChecked():
            customProj=[float(self.config[deposit][i]) for i in ['x','y','z']]
            self.data.translate(customProj, inOut=False)
            self.data.export(self.mLineEdit.text())
            self.data.translate(customProj)
        else :
            self.data.export(self.mLineEdit.text())

        self.mLabel.setText(self.tr('Database correctly exported.'))
        self.mCancelButton.setEnabled(True)
        self.mProgressBar.setMaximum(100)
        self.mProgressBar.setValue(100)
        self.mCancelButton.setEnabled(True)
