
import os

from PyQt5 import uic, QtCore, QtWidgets
from qgis.core import QgsProject, QgsLayoutExporter
from qgis.gui import QgsFileWidget

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'export_atlas.ui'))


class ExportAtlas(QtWidgets.QDialog, FORM_CLASS):
    def __init__(self, parent=None):
        """Constructor."""
        super(ExportAtlas, self).__init__(parent)
        # Set up the user interface from Designer.
        # After setupUI you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)

        self.mAtlasOutput.setStorageMode( QgsFileWidget.SaveFile )
        self.mAtlasOutput.setFilter("*.pdf")
        defaultPath = os.path.join(QtCore.QDir().home().path(), "Desktop")
        self.mAtlasOutput.setDefaultRoot( defaultPath )
        self.mAtlasOutput.setFilePath( os.path.join(defaultPath,
                                                    "export_atlas.pdf" ) )
        self.buttonBox.accepted.connect(self.exportAtlas)


    def __export_atlas(self, output_file, layout_name = 'export_niveaux'):

        project = QgsProject.instance()

        layout = project.layoutManager().layoutByName(layout_name)

        if layout:
            exporter = QgsLayoutExporter(layout)
            settings = QgsLayoutExporter.PdfExportSettings()
            settings.dpi = 100
            settings.rasterizeWholeImage = True
            err = exporter.exportToPdf(layout.atlas(), output_file, settings)

            return err

        return [QgsLayoutExporter.FileError, '']

    def exportAtlas(self):

        output_file = self.mAtlasOutput.filePath()
        if not os.path.exists(os.path.dirname(output_file)):
            QtWidgets.QMessageBox.warning(None, self.tr("Warning"), self.tr("Output destination does not exists!"))
            return
        if output_file:
            err = self.__export_atlas(output_file)

            msgBox = QtWidgets.QMessageBox()
            msgBox.setTextFormat(QtCore.Qt.RichText)

            if err[0] == QgsLayoutExporter.Success:
                msgBox.setText(self.tr("Atlas successfully exported to <a href='file:///{0}'>{0}</a>".format(output_file)))
                msgBox.exec_()
            else:
                msgBox.setText(self.tr(f"Error during export atlas: {err[1]}"))
                msgBox.exec_()

