# -*- coding: UTF-8 -*-

from __future__ import print_function
from OpenGL.GL import *
from OpenGL.GL import shaders

from OpenGL import GLU

from qgis.gui import *
from qgis.core import *

from PyQt5.QtOpenGL import QGLWidget, QGLFormat
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5 import uic

from qgis.gui import *
from qgis.core import *

import os
import re
import numpy

from .camera import Camera
from .scene import Scene
from .viewer_controls import ViewerControls

SLIDER_THICKNESS = 80


class CtrlCraftViewer3D(QGLWidget):

    def __del__(self):
        print('Viewer3d del')

    def __init__(self, conn_info=None, parent=None):
        super(CtrlCraftViewer3D, self).__init__(parent)
        self.setFocusPolicy(Qt.StrongFocus)
        self.scene = None
        self.mouseEventOnControl = False
        self.__param = {
                "slab": True,
                "xmax_cut": 100,
                "ymax_cut": 100,
                "xmin_cut": 0,
                "ymin_cut": 0,
                "panel": '',
                "blast": '',
                "radiometrySelection": ''
                }
        self.resetScene(conn_info)

    def close(self):
        if self.scene:
            self.scene.close()

    def refreshData(self, resetCut=True):
        if self.scene:
            self.resetScene(self.scene.conn_info, False, resetCut, self.__param['panel'], self.__param['blast'], '')
            self.update()

    def resetScene(self, conn_info, resetCamera=True, resetCut=True, panel='', blast='', radSel=''):
        self.__param['panel'] = panel
        self.__param['blast'] = blast
        self.__param['radiometrySelection'] = radSel
        if resetCut == True:
            self.__param['xmax_cut'] = 100
            self.__param['ymax_cut'] = 100
            self.__param['xmin_cut'] = 0
            self.__param['ymin_cut'] = 0
        if conn_info:
            self.scene and self.scene.setParent(None)
            self.scene = Scene(conn_info, self.__param, self)
            if resetCamera and self.scene.center:
                at = self.scene.center
                ext_y = self.scene.extent[3] - self.scene.extent[1]
                eye = at + QVector3D(0, -1.5 * ext_y, 0.5 * ext_y)
                self.camera = Camera(eye, at)

        else:
            self.scene and self.scene.setParent(None)
            self.scene = None
            self.camera = Camera(QVector3D(10, 10, -10), QVector3D(0, 0, 0))
        self.update()

    def resizeGL(self, width, height):
        height = 1 if not height else height
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        GLU.gluPerspective(45.0, float(width) / height, 100, 100000)
        glMatrixMode(GL_MODELVIEW)

    def paintGL(self, context=None, camera=None):
        context = context or self
        glClearColor(.9, .9, .9, 1.0)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        c = camera or self.camera
        dist = (c.at - c.eye).length()
        GLU.gluPerspective(45.0, float(context.width()) / context.height(), 0.01 * dist, 100 * dist)

        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)

        GLU.gluLookAt(c.eye.x(), c.eye.y(), c.eye.z(),
                      c.at.x(), c.at.y(), c.at.z(),
                      c.up.x(), c.up.y(), c.up.z())

        leftv = QVector3D.crossProduct(c.up, c.at - c.eye).normalized()
        upv = QVector3D.crossProduct(c.at - c.eye, leftv).normalized()

        to = (c.at - c.eye).normalized()

        glLightfv(GL_LIGHT0, GL_POSITION, [c.eye.x(), c.eye.y(), c.eye.z(), 1])

        if self.scene:
            self.scene.rendergl(leftv, upv, c.eye, context.height(), context)
        else:
            # Draw Test Cube (multiple quads)
            glEnable(GL_DEPTH_TEST)
            glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE)
            glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)
            glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  [1., 0., 0., 1.])
            glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  [1., 0., 0., 1.])
            glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, [1., 1., 1., 1.])
            glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50)

            glBegin(GL_QUADS)

            glNormal3f(0, 1, 0)
            glVertex3f( 1.0, 1.0,-1.0)
            glNormal3f(0, 1, 0)
            glVertex3f(-1.0, 1.0,-1.0)
            glNormal3f(0, 1, 0)
            glVertex3f(-1.0, 1.0, 1.0)
            glNormal3f(0, 1, 0)
            glVertex3f( 1.0, 1.0, 1.0)

            glNormal3f(0, 0, 1)
            glVertex3f( 1.0, 1.0, 1.0)
            glNormal3f(0, 0, 1)
            glVertex3f(-1.0, 1.0, 1.0)
            glNormal3f(0, 0, 1)
            glVertex3f(-1.0,-1.0, 1.0)
            glNormal3f(0, 0, 1)
            glVertex3f( 1.0,-1.0, 1.0)

            glNormal3f(1, 0, 0)
            glVertex3f( 1.0, 1.0,-1.0)
            glNormal3f(1, 0, 0)
            glVertex3f( 1.0, 1.0, 1.0)
            glNormal3f(1, 0, 0)
            glVertex3f( 1.0,-1.0, 1.0)
            glNormal3f(1, 0, 0)
            glVertex3f( 1.0,-1.0,-1.0)

            glEnd()

        # Draw controls on view
        self.draw2DControls(context.width(), context.height())

    def draw2DControls(self, width, height):
        # TODO move control function to specific file (controls.py) ?
        # Init 2D mode
        glLoadIdentity()
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0.0, width, 0.0, height, 0.0, 1.0)
        glMatrixMode (GL_MODELVIEW)
        glLoadIdentity()

        # Draw rectangles
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0.0, 1.0, 0.0, 6);
        self.draw_rect(0, 0, width, SLIDER_THICKNESS)

        glColor4f(0.0, 1.0, 0.0, 0.6);
        self.draw_rect(0, 0, SLIDER_THICKNESS, height)

    def draw_rect(self, x, y, width, height):
        glBegin(GL_QUADS)
        glVertex2f(x, y)
        glVertex2f(x + width, y)
        glVertex2f(x + width, y + height)
        glVertex2f(x, y + height)
        glEnd()

    #def setXCutPlane(self, value):
        #self.__param["x_cut"] = value
        #self.update()
    def setCutPlane(self, mode, delta_x, delta_y):

        if mode[0] == 'x':
            delta = 2.5 if delta_x>0 else -2.5
        else:
            delta = 2.5 if delta_y>0 else -2.5

        self.__param[mode] += delta

        min_bound={
                "xmax_cut": self.__param['xmin_cut'] + 1,
                "xmin_cut": 0,
                "ymax_cut": self.__param['ymin_cut'] + 1,
                "ymin_cut": 0
                }

        max_bound={
                "xmax_cut": 100,
                "xmin_cut": self.__param['xmax_cut'] - 1,
                "ymax_cut": 100,
                "ymin_cut": self.__param['ymax_cut'] - 1
                }

        self.__param[mode] = max(min_bound[mode], min(max_bound[mode], self.__param[mode]))

        self.refreshData(False)
        self.update()

    def setYCutPlane(self, value):
        self.__param["y_cut"] = value
        self.update()

    def mouseMoveEvent(self, event):
        delta_x = float(event.x() - self.oldx) / self.width()
        delta_y = float(self.oldy - event.y()) / self.height()

        # Check if event is over the 3D view, or over the controls
        zone = self.controlZone(event)
        if zone and self.mouseEventOnControl:
            self.setCutPlane(zone, delta_x, delta_y)
        else:
            self.camera.move(delta_x, delta_y, event.buttons(), event.modifiers())
        self.oldx = event.x()
        self.oldy = event.y()
        self.update()

    def wheelEvent(self,event):
        delta = 0.1
        if event.angleDelta().y() > 0:
            delta = -delta

        self.camera.move(0, delta, event.buttons(), event.modifiers())
        self.update()

    def mousePressEvent(self, event):
        # TODO check if this works with touch screens
        self.mouseEventOnControl = False
        if self.controlZone(event):
            self.mouseEventOnControl = True

        self.oldx = event.x()
        self.oldy = event.y()
        self.oldxCut = event.x()
        self.oldyCut = event.y()

    #def mouseReleaseEvent(self, event):
        #if self.mouseEventOnControl:
            #self.cutPlane(event)
        #self.mouseEventOnControl = False

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Space:
            self.camera.reset()
            self.update()

    def controlZone(self, event):
        x, y = event.x(), event.y()
        w, h = self.width(), self.height()
        if y > h - SLIDER_THICKNESS and x > w/2:
            return "xmax_cut"
        elif y > h - SLIDER_THICKNESS and x < w/2:
            return "xmin_cut"
        elif x < SLIDER_THICKNESS and y < h/2:
            return "ymax_cut"
        elif x < SLIDER_THICKNESS and y > h/2:
            return "ymin_cut"
        else:
            return None


class ViewerWindow(QMainWindow):
    def __init__(self, conn_info=None, parent=None):
        super(ViewerWindow, self).__init__(parent)
        self.resize(800,600)
        self.viewer = Viewer3d(conn_info, self)
        self.viewer.show()
        self.setCentralWidget(self.viewer)
        #self.controls = QDockWidget(self)
        #self.controls.setWidget(ViewerControls(self.viewer, self))
        #self.addDockWidget(Qt.RightDockWidgetArea, self.controls)


if __name__ == "__main__":

    from PyQt5.QtWidgets import *
    from PyQt5.QtCore import *
    import sys

    app = QApplication(sys.argv)

    QCoreApplication.setOrganizationName("QGIS")
    QCoreApplication.setApplicationName("QGIS2")

    win = ViewerWindow()
    if len(sys.argv) > 3:
        win.viewer.resetScene(sys.argv[1], panel=sys.argv[2], blast=sys.argv[3], radSel='')
    win.show()

    sys.exit(app.exec_())
